﻿using System;

namespace PV178.Tutorial6.Homework
{
    /// <summary>
    /// Represents a priority queue of people.
    /// </summary>
    public interface IPersonPriorityQueue
    {

        /// <summary>
        /// The QueueEmpty event is fired when last person is removed 
        /// from the queue and queue becomes empty.
        /// </summary>
        event PersonPriorityQueueDelegate QueueEmpty;


        /// <summary>
        /// Indicates that queue is empty.
        /// </summary>
        /// <value><c>true</c> if this queue is empty; otherwise, <c>false</c>.</value>
        bool IsEmpty { get; }


        /// <summary>
        /// Gets the number of people currently in the queue.
        /// </summary>
        /// <remarks>
        /// Retrieving the value of this property is an O(1) operation.
        /// </remarks>
        /// <value>The number of people in the queue.</value>
        int Count { get; }


        /// <summary>
        /// Gets the highest priority for a person in  the queue.
        /// </summary>
        /// <value>The highest priority for a person in the queue.</value>
        int HighestPriority { get; }


        /// <summary>
        /// Adds a person to the queue after all people with same or higher priority.
        /// </summary>
        /// <param name="person">The person to add to the priority queue. The value cannot be <c>null</c>.</param>
        /// <exception cref="ArgumentNullException">The person was <c>null</c>.</exception>
        void Enqueue(Person person);


        /// <summary>
        ///   Removes and returns the first person with the highest priority from
        ///   the beginning of the queue.
        /// </summary>
        /// <returns>
        ///   The object with the highest priority from the beginning  of the 
        ///   queue that is removed.
        /// </returns>
        /// <exception cref="InvalidOperationException">The priority queue is empty.</exception>
        Person Dequeue();


        /// <summary>
        /// Removes all people from priority queue.
        /// </summary>
        /// <remarks>
        /// Count is set to zero and maximum capacity is unchanged.
        /// </remarks>
        void Clear();
    }
}