﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace PV178.Tutorial6.Homework
{
    public class PersonPriorityQueue : IPersonPriorityQueue
    {
        public event PersonPriorityQueueDelegate QueueEmpty;


        // one queue for each possible priority
        private readonly SortedList<int, Queue<Person>> queues;

        private IPriorityPlanner planner;

        private int count = 0;


        public int Count
        {
            get { return count; }
            private set
            {
                if (value != count)
                {
                    count = value;
                    if( count == 0 )
                    {
                      OnQueueEmpty();
                    }
                }
            }
        }

        public bool IsEmpty
        {
            get { return Count == 0; }
        }

        public int HighestPriority
        {
            get { return queues.Count - 1; }
        }


        /// <summary>
        /// Creates a new priority queue with priorities in range from 0 to
        /// <paramref name="maxPriority"/>.
        /// </summary>
        /// <param name="maxPriority">The highest possible priority of person in the queue.</param>
        /// <exception cref="ArgumentOutOfRangeException"><c>maxPriority</c> must be greater than or equal to zero.</exception>
        public PersonPriorityQueue(int maxPriority)
        {
            if (maxPriority < 0)
                throw new ArgumentOutOfRangeException("maxPriority");

            queues = new SortedList<int, Queue<Person>>();
            for (int priority = 0; priority <= maxPriority; priority++)
            {
                queues[priority] = new Queue<Person>();
            }
            planner = new PriorityPlanner();
        }


        public void Enqueue(Person person)
        {
            if (person == null)
                throw new ArgumentNullException("item");
            
            int priority = planner.ComputePriority(person);
            queues[priority].Enqueue(person);
            Count++;
        }


        public Person Dequeue()
        {
            if (IsEmpty)
                throw new InvalidOperationException("Priority queue is empty.");

            for (int priority = queues.Count - 1; priority >= 0; priority--)
            {
                if (queues[priority].Count > 0)
                {
                    Person result = queues[priority].Dequeue();
                    Count--;
                    return result;
                }
            }

            // compiler requires following line
            throw new InvalidOperationException("Empty queue attepted to dequeue.");
        }

        public void Clear()
        {
            foreach (var queue in queues)
            {
                queue.Value.Clear();
            }
            Count = 0;
        }

        private void OnQueueEmpty()
        {
            PersonPriorityQueueDelegate handler = QueueEmpty;
            if (handler != null)
            {
                handler(this);
            }
        }
    }
}
